// BaseOpenTKControl.axaml.cs

using System;
using Avalonia;
using Avalonia.Controls;
using Avalonia.Markup.Xaml;
using Avalonia.OpenGL;
using Avalonia.OpenGL.Controls;
using OpenTK;
using OpenTK.Graphics;
using OpenTK.Graphics.OpenGL;

namespace AvaloniaOpenTK.Controls;

public partial class BaseOpenTKControl : OpenGlControlBase
{

    private int _vao;
    private int _vbo;

    private readonly float[] _verts =
    {
        0.0f, 0.5f,
        -0.5f, -0.5f,
        0.5f, -0.5f
    };

public BaseOpenTKControl()
    {
        InitializeComponent();
    }
    
    protected override void OnOpenGlInit(GlInterface gl)
    {
        GLLoader.LoadBindings(new AvaloniaBindingsContext(gl));
        GL.ClearColor(new(0.2f, 0.4f, 0.8f, 1.0f));
        
        _vao = GL.GenVertexArray();
        GL.BindVertexArray(_vao);
        
        _vbo = GL.GenBuffer();
        GL.BindBuffer(BufferTarget.ArrayBuffer, _vbo);
        
        GL.BufferData(BufferTarget.ArrayBuffer, _verts.Length * sizeof(float), _verts, BufferUsage.StaticDraw);
        
        GL.EnableVertexAttribArray(0);
        GL.VertexAttribPointer(0, 2, VertexAttribPointerType.Float, false, 2 * sizeof(float), 0);
        
        GL.BindVertexArray(0);
    }

    protected override void OnOpenGlRender(GlInterface gl, int fb)
    {
        var width = (int)Bounds.Width;
        var height = (int)Bounds.Height;
        GL.Viewport(0, 0, width, height);
        GL.Clear(ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);
        
        GL.BindVertexArray(_vao);
        GL.DrawArrays(PrimitiveType.Triangles, 0, 3);
        
        GL.BindVertexArray(0);
    }

    protected override void OnOpenGlDeinit(GlInterface gl)
    {
        GL.DeleteBuffer(_vbo);
        GL.DeleteVertexArray(_vao);
    }
    
    private class AvaloniaBindingsContext : IBindingsContext
    {
        private readonly GlInterface _gl;
        
        public AvaloniaBindingsContext(GlInterface gl)
        {
            _gl = gl;
        }
        
        public IntPtr GetProcAddress(string procName)
        {
            return _gl.GetProcAddress(procName);
        }
    }
}

